#include "Polygone.h"
#include "OutilsMaths.h"
#include "FoncteursUtilitaires.h"
#include <algorithm>
#include "Triangle.h"

int Polygone::CompterPrimitives () const throw ()
{
   unsigned int Cpt = 1; // milieu de 0 et de 2
   int NbPrim = 0;
   while (Cpt + 1 < m_Points.size ())
   {
      if (CalculerAngle (m_Points[Cpt-1], m_Points[Cpt], m_Points[Cpt+1]) <= PI)
         NbPrim++;
      Cpt++;
   }
   return NbPrim;
}

namespace
{
   typedef std::pair <const pt *, const int> PaireUtile;
   bool ComparateurDereference (const PaireUtile *a, const PaireUtile *b)
   {
      return a->first < b->first;
   }
}

bool TrouverPtDansTriangle (const Triangle &t, const std::vector<pt> &v)
{
   bool Trouve = false;
   std::vector<pt>::const_iterator itt = v.begin ();
   while (itt != v.end () && !Trouve)
   {
      pt p (*itt++);
      if (t.PointDansPolygone (p)) Trouve = true;
   }
   return Trouve;
}

Triangle CouperPrimitive (std::vector<pt> &v)
{
   std::vector<pt>::iterator ptA = v.begin();
   std::vector<pt>::iterator ptB = ptA;
   ::ProgressionCirculaire (ptB, v.begin (), v.end ());
   std::vector<pt>::iterator ptC = ptB;
   ::ProgressionCirculaire (ptC, v.begin (), v.end ());

   bool TriangleVide;
   do
   {
      //
      // On cherche un Triangle qui peut tre coup...
      //
      while (ADroite (*ptA, *ptB, *ptC))
      {
         ::ProgressionCirculaire (ptA, v.begin (), v.end ());
         ::ProgressionCirculaire (ptB, v.begin (), v.end ());
         ::ProgressionCirculaire (ptC, v.begin (), v.end ());
      }
      //
      // On a un angle interne convenable. Y a-t-il un point dedans?
      //
      TriangleVide = !TrouverPtDansTriangle (Triangle (*ptA, *ptB, *ptC), v);
      if (!TriangleVide)
      {
         ::ProgressionCirculaire (ptA, v.begin (), v.end ());
         ::ProgressionCirculaire (ptB, v.begin (), v.end ());
         ::ProgressionCirculaire (ptC, v.begin (), v.end ());
      }
   }
   while (!TriangleVide);
   Triangle t (*ptA, *ptB, *ptC);
   v.erase (ptB);
   return t;
}

std::vector<Triangle*> Polygone::EnumererPrimitives () const throw ()
{
   //
   // On garde un vecteur de Triangle pour lister les primitives dcoupes
   //
   std::vector<Triangle*> vPrimitives;
   //
   // On travaille avec une copie des points du polygone, pour ne pas le
   // modifier inutilement
   //
   std::vector<pt> vTemp = m_Points;

   //
   // On coupe des oreilles, une  une, jusqu' ce qu'il n'en reste plus
   // une seule.  la fin, on insre le seul triangle restant dans la liste
   // des primitives
   //
   while (vTemp.size () > Triangle::NB_PTS)
      vPrimitives.push_back (new Triangle (CouperPrimitive (vTemp)));
   vPrimitives.push_back (new Triangle (vTemp[0], vTemp[1], vTemp[2]));
   return vPrimitives;
}

bool Polygone::PointDansPolygone (const pt &p) const throw ()
{
   //
   // Les bordures, incluant les sommets, ne sont pas considrs dedans
   //
   if (std::find (m_Points.begin (), m_Points.end (), p) != m_Points.end ()) return false;
   //
   // On trace une demi-ligne dans une direction arbitraire
   //  partir de p; on vrifie le nombre d'intersections
   // entre le polygone (this) et cette demi-ligne. Le point
   // p est dans le polygone (this) si et seulement si le
   // nombre d'intersections est impair.
   //
   // Prsomption: le triangle et p sont tous  plat sur
   // l'axe des z (pour simplifier).
   //
   // Notre demi-ligne ira de p.x vers l'infini positif, et
   // restera sur le mme point de l'axe des y. On dira donc
   // que p croise le segment v0v1 si p.y se trouve entre v0.y
   // et v1.y, et si p.x < v0.x ou p.x < v1.x
   //
   std::vector<pt>::const_iterator itt0 = m_Points.begin ();
   std::vector<pt>::const_iterator itt1 = m_Points.begin ();
   itt1 ++;
   size_t NbIntersections = 0;

   while (itt1 != m_Points.end ())
   {
      pt p0 (*itt0), p1 (*itt1);
      if (OutilsMaths::Entre (p.y, itt0->y, itt1->y))
      {
         // Calculer la pente, trouver si le x de p.y est > p.x
         if (OutilsMaths::Egaux (itt0->x, itt1->x) && p.x < itt0->x)
            NbIntersections++;
         else
         {
            double Pente = (itt1->y - itt0->y) / (itt1->x - itt0->x);
            double Deplacement = itt1->y - Pente * itt1->x; // ou itt0...
            double PtX = (p.y - Deplacement) / Pente;
            if (p.x < PtX) NbIntersections++;
         }
      }
      itt0++;
      itt1++;
   }
   itt1 = m_Points.begin ();
   if (OutilsMaths::Entre (p.y, itt0->y, itt1->y))
   {
      // Calculer la pente, trouver si le x de p.y est > p.x
      if (OutilsMaths::Egaux (itt0->x, itt1->x) && p.x < itt0->x)
         NbIntersections++;
      else
      {
         double Pente = (itt1->y - itt0->y) / (itt1->x - itt0->x);
         double Deplacement = itt1->y - Pente * itt1->x; // ou itt0...
         double PtX = (p.y - Deplacement) / Pente;
         if (p.x < PtX) NbIntersections++;
      }
   }
   return NbIntersections % 2 != 0;
}