#include "pt.h"

std::ostream &operator<< (std::ostream &o, const pt &p)
{
   o << "x: " << p.x << ", y: " << p.y << ", z: " << p.z;
   return o;
}

float CalculerAngleEntreVecteurs (const pt &p1, const pt &p2) throw (VecteurLongueurNulle)
{
   if (!p1.lg () || !p2.lg ()) throw VecteurLongueurNulle ();
   return std::acos (p1.ProduitScalaire (p2) / (p1.lg () * p2.lg ()));
}

float CalculerAngle (const pt &A, const pt &B, const pt &C) throw (VecteurLongueurNulle)
{
   pt v0 (A - B), v1 (C - B);
   return CalculerAngleEntreVecteurs (v0, v1);
}

//
// Virage horaire
//
// Le vecteur (p0p1) se trouve accessible par un virage horaire 
// partir du vecteur (p0p2) si et seulement si le produit scalaire
// (p1-p0)*(p2-p0)>0
//
bool VirageHoraire (const pt &p0, const pt &p1, const pt &p2)
{
//   return pt (p1 - p0).ProduitScalaire (pt (p2 - p0)) > 0;
   return pt (p1 - p0).CrossProduct (pt (p2 - p0)) > 0;
}

//
// Virage antihoraire
//
// Le vecteur (p0p1) se trouve accessible par un virage antihoraire 
// partir du vecteur (p0p2) si et seulement si le produit scalaire
// (p1-p0)*(p2-p0)<0
//
bool VirageAntihoraire (const pt &p0, const pt &p1, const pt &p2)
{
   return pt (p1 - p0).CrossProduct (pt (p2 - p0)) < 0;
}

bool AGauche (const pt &p0, const pt &p1, const pt &p2)
{
   return VirageHoraire (p0, p1, p2);
}

bool ADroite (const pt &p0, const pt &p1, const pt &p2)
{
   return VirageAntihoraire (p0, p1, p2);
}



